/* event.c -- event management stuff
 *
 * author: hackbard@hackdaworld.dyndns.org
 *
 */

#include "event.h"

int event_init(t_event *event,int outfd) {

  dprintf(outfd,"[event] initializing event system ...\n");

  event->maxfd=0;

  event->outfd=outfd;

  FD_ZERO(&(event->rfds));
  FD_ZERO(&(event->rfds_o));
  FD_ZERO(&(event->wfds));
  FD_ZERO(&(event->wfds_o));

  return E_SUCCESS;
}

int event_set_timeout(t_event *event,int sec,int usec) {

  event->timeout.tv_sec=sec;
  event->timeout.tv_usec=usec;

  return E_SUCCESS;
}

int event_set_immediately(t_event *event,unsigned char value) {

  if(value==IMM_TRUE)
    event->immediately=value;

  return E_SUCCESS;
}

int event_math(int fd,t_event *event,char rw,char ar) {

  if(ar==ADD) {
    if(rw==READ) FD_SET(fd,&(event->rfds_o));
    else FD_SET(fd,&(event->wfds_o));
    if(fd>event->maxfd) event->maxfd=fd;
  } else {
    if(rw==READ) FD_CLR(fd,&(event->rfds_o));
    else FD_CLR(fd,&(event->wfds_o));
    if(fd==event->maxfd) event->maxfd-=1;
  }

  dprintf(event->outfd,"[event] %c fd %d to %c file descriptor set\n",
          ar==ADD?'+':'-',fd,rw==READ?'r':'w');

  return E_SUCCESS;
}

int event_start(t_event *event,void *ptr,
                int (*callback0)(t_event *event,void *ptr),
                int (*callback1)(t_event *event,void *ptr)) {

  int s_ret;
  struct timeval tv;
  struct timeval *tv_ptr;

  /* switch on event system */
  event->status=ENABLED;

  while(event->status&ENABLED) {

    event->rfds=event->rfds_o;
    event->wfds=event->wfds_o;
    tv=event->timeout;

    if((tv.tv_sec==0)&&(tv.tv_usec==0)&&(!(event->immediately))) tv_ptr=NULL;
    else tv_ptr=&tv;

    if((s_ret=select(event->maxfd+1,&(event->rfds),&(event->wfds),NULL,
                     tv_ptr))==-1) {
      perror("[event] select call");
      return E_ERROR;
    }

    if(s_ret) {
      /* 1+ fd(s) available for reading/writing */
      if(callback0(event,ptr)==CALLBACK_ERROR) {
        dprintf(event->outfd,"[event] callback0 returned error\n");
        return E_ERROR;
      }
    }

    /* usual stuff to be executed */
    if(callback1(event,ptr)==CALLBACK_ERROR) {
      dprintf(event->outfd,"[event] callback returned error\n");
      return E_ERROR;
    }

  }

  return E_SUCCESS;
}

int event_check(t_event *event,int fd) {

  if(FD_ISSET(fd,&(event->wfds))|FD_ISSET(fd,&(event->rfds))) return E_FD_YES;
  else return E_FD_NO;

}

int event_stop(t_event *event) {

  dprintf(event->outfd,"[event] shutdown\n");
  event->status=DISABLED;

  return E_SUCCESS;
}
